/********************************************************************************/
/*																				*/
/*	host_hsb.c																	*/
/*																				*/
/*	Example host program for sending and receiving messages using HSB.			*/
/*																				*/
/*	The program sends a message containing 10 data bytes to a module in slot 1.	*/
/*	The	program then expects to receive a message from the same slot, with the	*/
/*	same data bytes sent in the message. The program then checks the received	*/
/*	data against the data that was sent.										*/
/*																				*/
/*																				*/
/*	(c) 2001 Hunt Engineering													*/
/*	    All Rights Reserved														*/
/*																				*/
/*	Version	  Date   Version Author	 Comments									*/
/*	-------	  ----   --------------	 --------									*/
/*																				*/
/* 	  1.0	14-03-00	   RW		 First written.								*/
/*																				*/
/********************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <stddef.h>
#include <conio.h>
#include <time.h>
#include <dos.h>
#include <signal.h>

#include "stdafx.h"
#include "heapi.h"


/* The following definition sets the board string of the carrier being used */
/* For this example the string is set up for a HEPC8 */
#define DEVICE  "hep8a\0"

/* The following definition sets the slot number with which to communicate */
#define SLOT	1

/* The following definition sets the number of data bytes in each message */
#define SIZE	10


/* Function Prototypes */
int  Initialise_Host_HSB(char *dev_name, HE_WORD board);
void Error(HE_DWORD errorcode);
void Print_Win32_Error(HE_DWORD errorcode);


/* Global Variables */
HE_HANDLE   hDevice			= NULL;
HE_IOSTATUS ReadIoStatus	= NULL;
HE_IOSTATUS WriteIoStatus	= NULL;
HE_DWORD    Status;



int main(int argc, char *argv[])
{
	int i,j,k;
	int status;
	int errors;
	char dev_name[] = DEVICE;
	HE_BYTE  message_type;
	HE_WORD  board;
	HE_DWORD bytes_read;
	HE_BYTE  send[SIZE];
	HE_BYTE  receive[SIZE];


	/* Initialise the HSB interface on this board */
	board = 0;
	status = Initialise_Host_HSB(dev_name,board);
	if (status < 0) {
		printf("Error: call to Initialise_Host_HSB failed\n");
		exit(0);
	}


	i = 0;
	for (k=0;k<10;k++) {
		printf("\n\nPress return to send a message to slot %d\n",SLOT);
		getchar();
		
		for (j=0;j<SIZE;j++) {
			send[j] = i++;
			receive[j] = 0;
			if (i > 255) i = 0;
		}

		/* For this example, an arbitrary message type value of 0xaa is used */
		message_type = 0xaa;

		/* The following function call is used to send a message over HSB */
		Status = HeHSBSendMessage(hDevice,message_type,SLOT,send,SIZE,0);
		if (Status != 0) {
			printf("Error: failed to successfully send message to slot %d\n",SLOT);
			exit(0);
		}

		printf("Press return to receive message from slot %d\n",SLOT);
		getchar();

		/* The following function call is used to receive a message over HSB */
		Status = HeHSBReceiveMessage(hDevice,&message_type,SLOT,receive,SIZE,&bytes_read,0);
		if (Status != 0) {
			printf("Error: failed to successfully receive message from slot %d\n",SLOT);
			exit(0);
		}

		if (bytes_read != SIZE) {
			printf("Error: expected %d data bytes, received %d\n",SIZE,bytes_read);
			exit(0);
		}

		printf("Checking received data\n");
		errors = 0;
		for (j=0;j<SIZE;j++) {
			if (send[j] != receive[j]) {
				printf("Error: Receive[%d] = %02x\n",j,receive[j]);
				printf("\texpected %02x\n",send[i]);
				errors++;
				getchar();
			}
		}
		if (errors > 0) exit(0);
		printf("Successfully sent and received HSB messages\n");
		printf("Press return to continue\n");
		getchar();
	} 

	return(0);
}


/********************************************************************************/
/*																				*/
/*	Initialise_Host_HSB															*/
/*																				*/
/*	This function sets up the host HSB interface of each board in the system.	*/
/*																				*/
/********************************************************************************/
int Initialise_Host_HSB(char *dev_name, HE_WORD board)
{

	/* Check parameters */
	if ((board < 0) || (board > 15)) {
		printf("Error: board ID is out of range (0 to 15)\n");
		return(-1);
	}

	hDevice	  = NULL;
	ReadIoStatus  = NULL;
	WriteIoStatus = NULL;

	Status = HeOpen1(dev_name, board, "hsb\0", &hDevice);
	if (Status != HE_OK) return(1);
	
	Status = HeInitIoStatus(hDevice, &ReadIoStatus);
	if (Status != HE_OK) Error(HeGetLastOsError(hDevice));

	Status = HeInitIoStatus(hDevice, &WriteIoStatus);
	if (Status != HE_OK) Error(HeGetLastOsError(hDevice));

	return(0);
}


/********************************************************************************/
/*																				*/
/*	Error																		*/
/*																				*/
/*	General Error Routine														*/
/*																				*/
/********************************************************************************/
void Error(HE_DWORD errorcode)
{
	char text[120];


	printf("In error routine\n");
	HeErr2Text(Status, text);
	printf("Failed - %s, OsError = %ld", text, errorcode);
	Print_Win32_Error(errorcode);
	printf("\n");

	Status = HeClose(&hDevice);
	exit(1);
}


/********************************************************************************/
/*																				*/
/*	Print_Win32_Error															*/
/*																				*/
/*	Win32 Error Function														*/
/*																				*/
/********************************************************************************/
void Print_Win32_Error(HE_DWORD errorcode)
{
#ifdef _WIN32			/* This level of error code is only on WIN32 */
						/* don't even compile it if not WIN32!*/
	LPVOID lpMsgBuf;

	if(errorcode != 0) {
		printf(" (%s)\n", strerror(errorcode));

		FormatMessage(FORMAT_MESSAGE_ALLOCATE_BUFFER | FORMAT_MESSAGE_FROM_SYSTEM,
			NULL,
			errorcode, 
			MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT), /* Default language*/
			(LPTSTR) &lpMsgBuf,
			0,
			NULL);
		printf("Sys Error - %s", lpMsgBuf);
	}
#endif				/* end of WIN32 specific error codes*/
}

